
/*
 *
 * nalloc.h
 * 
 */

#ifndef _NALLOC_H
#define _NALLOC_H 1

#include <stdlib.h>
#include "primtypes.h"



/* Maximum number of structures allocated within a node */

#define NA_SLOTS 100

typedef size_t na_offset;
typedef int32_t na_slot_t,na_handle,slot_t,tag_t;



/*
 * Here is the actual internal structure that can be
 * used as a zram node
 */
#ifdef USE_ZRAM
#include <z_base.h>
#define ZRAM_HEADER Z_NODE_HEADER
#define ZALLOC(s,tag) (na_node_p)z_NewNode(tag,s)
#define ZREALLOC(p,s) (na_node_p)z_ReallocNode((z_NodeP)p,s);
#else 
#define ZRAM_HEADER long tag
#define ZREALLOC(p,s) memrealloc(p,s)
#define ZALLOC(s,tag) memalloc(s)
#define MINT int
#endif

/* descriptor for a chunk of memory */

struct na_node_struct;
typedef void (* na_func)(struct na_node_struct *, na_slot_t slot); 
typedef struct na_desc_struct{
  na_offset	offset;		/* Where to find it in the node */
  size_t	size;		/* How big is this chunk */
  bool		in_use;		/* Currently in use */
  bool		dangles;	/* Currently allocated outside the node */
  void		*ptr;		/* at the moment, only used when dangling */
  array_index_t fixup_tag;		/* fixup pointers */
  array_index_t destroy_tag;		/* call this routine when destroying */
  array_index_t wipe_aux_tag;		/* Wipe out aux pointers */
  array_index_t free_aux_tag;		/* Wipe out aux pointers */
  array_index_t report_tag;		/* tell me about this piece of memory */
} na_desc_t, *na_desc_p;
     

  
typedef struct na_node_struct{ 
  ZRAM_HEADER;
  bool frozen;		/* This is true is nothing dangles */
  bool destroyed;       /* For debugging. Should never be true on entry */
  size_t size;
  size_t limit;
  na_desc_t	chunk[NA_SLOTS];
  char mem[1];
} na_node_t, *na_node_p;


#ifdef MEMDEBUG
#define NODECHECK(s)  na_sanity_check(HERE,s)
#else
#define NODECHECK(s) /**/
#endif


#define NA_INITIAL_SIZE 128
#define NA_ALLOC_INCR 128
#define NA_ALLOC_MUL 1
/* Define an alignment that always works */
/* This should probably be autoconfed */
#ifndef MAX
#define MAX(a,b) (((a)>(b)) ? (a) : (b))
#endif
#define ALIGN_BYTES MAX(sizeof(long),sizeof(double))
#define ALIGN(b) (  ( ( (b) % ALIGN_BYTES)==0 ) ? (b) : \
		    ((long)(b)/ALIGN_BYTES+1)*ALIGN_BYTES )




/* prototypes */

void *na_access(na_node_p,na_slot_t);
void *na_realloc(na_node_p,na_slot_t,size_t);
void na_destroy(na_node_p);
na_node_p na_new(MINT);
na_node_p na_new1(MINT,size_t);

na_node_p na_alloc(na_node_p,na_slot_t,size_t);
void na_copy(na_node_p,na_node_p);
na_node_p na_freeze(na_node_p);
na_node_p na_compact(na_node_p);
size_t na_size(na_node_p);
void na_sanity_check(char*,na_node_p);
void na_fixup(na_node_p);
void na_cleanup(na_node_p);
void na_set_fixup(na_node_p,na_slot_t,array_index_t);
void na_set_destroy(na_node_p,na_slot_t,array_index_t);
void na_set_forget(na_node_p,na_slot_t,array_index_t);
void na_set_report(na_node_p,na_slot_t,array_index_t);
void na_set_wipe_aux(na_node_p,na_slot_t,array_index_t);
void na_set_free_aux(na_node_p,na_slot_t,array_index_t);
void na_report(na_node_p);
void na_free_aux(na_node_p);

void na_fatal(char *);

extern na_func na_jump_table[];

#define SLOT(v) SLOT_##v
#define RSLOT(s,t,e) s+t##_OFF_##e


/* Tags used for global jump table */

#define TAG(func) _TAG(func)
#define _TAG(func) TAG_##func

#define TAG_DECLARE(func) extern int TAG(func)
#endif
