#include "parser.h"

#include "combinat.h"
#include "error_message.h"
#include "gparams.h"
#include "nu_sign_set.h"
#include "startopt.h"
#include "statearray.h"
#include "runtime_options.h"
#include "output.h"
#include "profile.h"

int main(int argc, char **argv){

  params_p params;
  sarray_p chi,new_chi,bdy=0;
  int bases,new_bases;
  int i;

  int *indices,*elements;

  prof_init();
  params=malloc(sizeof(params_t));

  ro_process_args(argc,argv,params,SO_nonuniform);

  if (!parse_input(params))
    exit(0);
  
  if (params->contract==-1)
    err_fatal(HERE,"no contraction element specified");

  bases=binomial(params->n,params->r);

  chi=sa_new(bases,bases);
  for (i=0; i < bases; i++){
    sa_init(chi,i,SIGN_ANY);
  }
  for (i=0; i <nss_count(parser.basis_signs); i++){
    int bindex;
    nu_sign_t sigma;
    nu_signed_index_t si=nss_next(parser.basis_signs);
    
    bindex=nsi_index(si);
    sigma=nsi_sign(si);
    sa_init(chi,bindex,sigma);
  }

  new_bases=binomial(params->n-1,params->r-1);
  new_chi=sa_new(new_bases,new_bases);


  indices=malloc(params->r*sizeof(int));
  elements=malloc(params->r*sizeof(int));

  first_kset(params->n-1,params->r-1,indices);
  do {
    int i;
    nu_sign_t sigma;
    array_index_t bindex;

    /* claim: this transformation preserves lexicographic ordering 
       This justifies the use of the raw indices to represent the 
       basis index in the new oriented matroid.
    */
    for (i=0; i<params->r-1; i++){
      if (indices[i] < params->contract){
	elements[i]=indices[i];
      } else {
	elements[i]=indices[i]+1;
      }
    }
    elements[params->r-1]=params->contract;
    
    bindex=kset_cindex(params->n,params->r,elements);
    sigma=sa_read(chi,bindex);

    bindex=kset_cindex(params->n-1,params->r-1,indices);
    sa_init(new_chi,bindex,
	   sign_mulint(sigma,basis_parity(params->r,elements)));
  } while (next_kset(params->n-1,params->r-1,indices));

  params->n--;
  params->r--;
  output_chi(params,new_chi);
  return 0;
}


