/* common routines for initialization */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "startup.h"
#include "parser.h"
#include "nodedef.h"
#include "globaldef.h"
#include "nalloc.h"
#include "basis_equation.h"
#include "bindex_table.h"
#include "oms.h"
#include "hyperline.h"
#include "output.h"
#include "oms_tune.h"
#include "combinat.h"
#include "sa16_array.h"
#include "checkpoint.h"
#include "variable_choice.h"
#include "nu_signed_index.h"
#include "debug.h"
#ifdef USE_ZRAM
#include <z_lb.h>
#include <z_checkpoint.h>
#endif

#include "common_startup.h"


extern int oms_GlobalTag,oms_NodeTag;

static void read_chirotope(na_node_p, na_node_p, sarray_p);
static void read_boundary(na_node_p, na_node_p, sarray_p);




extern void local_fixup(na_node_p state){

#ifdef HLS_STARTUP
    hl_fixup(state,SLOT_hyp);
#endif

    sa_fixup(state,SLOT_pos_mask);
    sa_fixup(state,SLOT_neg_mask);

#ifdef OMS_STARTUP
    sa16_fixup(state,SLOT_pe_state);
#endif

    sa_fixup(state,SLOT_chirotope);

    sa_fixup(state,SLOT_boundary);
    rg_fixup(state,SLOT_ridge_graph); 

}



extern void global_fixup(na_node_p global){


  params_p params=na_access(global,SLOT_params);
  
  if (in_oms_startup || params->extra_pluecker)
    be_fixup(global,SLOT_pe);

  be_fixup(global,SLOT_bdy_eq);
  be_fixup(global,SLOT_pivots);
}



void SaveState(na_node_p state){

  lparams_p lparams=na_access(state,SLOT_lparams);
  lparams->depth++;

  rg_save(na_access(state,SLOT_ridge_graph));
  hl_save(na_access(state,SLOT_hyp));

  sa_save(na_access(state,SLOT_pos_mask));
  sa_save(na_access(state,SLOT_neg_mask));
  sa_save(na_access(state,SLOT_chirotope));

  sa_save(na_access(state,SLOT_boundary));
}


extern void RestoreState(search_state state){

  lparams_p lparams;

  NODECHECK(state);

  lparams=na_access(state,SLOT_lparams);

  lparams->depth--;

   rg_restore(na_access(state,SLOT_ridge_graph));
#ifdef OMS_STARTUP   
   sa16_restore(na_access(state,SLOT_pe_state));
#else
   hl_restore(na_access(state,SLOT_hyp));
#endif

   sa_restore(na_access(state,SLOT_pos_mask));
   sa_restore(na_access(state,SLOT_neg_mask));
   sa_restore(na_access(state,SLOT_chirotope));
   sa_restore(na_access(state,SLOT_boundary));
 }

