/* pm_both.c
   Enumeration of polyominoes. Common routines for reverse search and
   backtrack implementation.

   Ambros Marzetta
   
 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 17:14:52 +0100 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:52  bremner
 * committing patched version of zram source
 *
 * Revision 1.1  1998/06/17  18:17:42  marzetta
 * Initial revision
 *
 */

#include "pm_both.h"
#include <stdlib.h>

static int compare(const void *a, const void *b)
     /* Integer comparison, used by qsort */
{
  return *(int *)a - *(int *)b;
} /* compare */

boolean lexmax(int a[MAXN], int count, int xmax, int ymax)
{
  /* Return true if a[0..count-1] is lexicographically maximal compared to
     symmetric polyominoes. */
  int i, j, b[MAXN];

  qsort(a, count, sizeof(*a), compare);	/* sort points */

  for (i=count-1; i>=0; --i) b[i] = a[i];
  for (j = 0; j < 3; j++) {	/* rotate 3 times by 90 degrees */
    for (i=count-1; i>=0; --i)
      b[i] = ((j%2?xmax:ymax)-b[i]%MAXN) * MAXN + b[i]/MAXN;
    qsort(b, count, sizeof(*b), compare);
    for (i=count-1; i>=0; --i)
      if (b[i] > a[i]) return 0; /* return 0 if lexicographically greater */
      else if (b[i] < a[i]) break;
  }

  for (i=count-1; i>=0; --i) b[i] = a[i] / MAXN * MAXN + ymax-a[i] % MAXN;
				/* mirror */
  qsort(b, count, sizeof(*b), compare);
  for (i=count-1; i>=0; --i)
    if (b[i] > a[i]) return 0; /* return 0 if lexicographically greater */
    else if (b[i] < a[i]) break;
  
  for (j = 0; j < 3; j++) {	/* rotate 3 times by 90 degrees */
    for (i=count-1; i>=0; --i)
      b[i] = ((j%2?xmax:ymax)-b[i]%MAXN) * MAXN + b[i]/MAXN;
    qsort(b, count, sizeof(*b), compare);
    for (i=count-1; i>=0; --i)
      if (b[i] > a[i]) return 0; /* return 0 if lexicographically greater */
      else if (b[i] < a[i]) break;
  }

  return 1;
} /* lexmax */
